@@ -1,5 +1,6 @@
 """
-Defines the `command` decorator and the `CommandTree` class.
+Defines the :py:func:`@command<dollar_lambda.decorators.command>` decorator
+and the :py:class:`CommandTree<dollar_lambda.decorators.CommandTree>` class.
 """
 from __future__ import annotations
 
@@ -14,12 +15,12 @@ from typing import Any, Callable, Iterator, List, Optional, Type, TypeVar
 from pytypeclass import Monoid
 from pytypeclass.nonempty_list import NonemptyList
 
-from dollar_lambda import parser as parser_mod
+from dollar_lambda import parsers
 from dollar_lambda.args import _ArgsField
-from dollar_lambda.error import ArgumentError
-from dollar_lambda.parser import Parse, Parser, matches
+from dollar_lambda.data_structures import KeyValue, Output, Sequence
+from dollar_lambda.errors import ArgumentError
+from dollar_lambda.parsers import Parse, Parser, matches
 from dollar_lambda.result import Result
-from dollar_lambda.sequence import KeyValue, Output, Sequence
 
 A = TypeVar("A")
 B = TypeVar("B")
@@ -31,61 +32,81 @@ def _func_to_parser(
     exclude: Optional[List[str]],
     flip_bools: bool,
     help: Optional[typing.Dict[str, str]],
-    parsers: Optional[typing.Dict[str, Parser[Output]]],
+    parsers: Optional[typing.Dict[str, "Parser[Output] | List[Parser[Output]]"]],
     repeated: Optional[Parser[Output]],
+    prefix: Optional[str] = None,
 ) -> Parser:
     _exclude = [] if exclude is None else exclude
     _help = {} if help is None else help
     _parsers = {} if parsers is None else parsers
 
     types = typing.get_type_hints(func)  # see https://peps.python.org/pep-0563/
-    p = [
-        _parsers.get(
-            k,
-            _ArgsField(
-                name=k,
-                default=None if v.default == Parameter.empty else v.default,
-                help=_help.get(k),
-                type=types.get(k, str),
-            ),
-        )
-        for k, v in signature(func).parameters.items()
-        if k not in _exclude
-    ]
-    return _ArgsField.parser(*p, flip_bools=flip_bools, repeated=repeated)
+
+    def get_parsers():
+        for k, v in signature(func).parameters.items():
+            if k not in _exclude:
+                if k in _parsers:
+                    p = _parsers[k]
+                    if isinstance(p, list):
+                        if len(p) > 1 and v.kind not in (
+                            Parameter.VAR_KEYWORD,
+                            Parameter.VAR_POSITIONAL,
+                        ):
+                            raise RuntimeError(
+                                f"Assigning multiple parsers to a non-variational argument '{v.name}' will result in "
+                                f"binding collisions. "
+                            )
+                        yield from p
+                    else:
+                        yield p
+                else:
+                    yield _ArgsField(
+                        name=k if prefix is None else f"{prefix}.{k}",
+                        default=None if v.default == Parameter.empty else v.default,
+                        help=_help.get(k),
+                        type=types.get(k, str),
+                    )
+
+    return _ArgsField.parser(*get_parsers(), flip_bools=flip_bools, repeated=repeated)
 
 
 def command(
     flip_bools: bool = True,
     help: Optional[typing.Dict[str, str]] = None,
-    parsers: Optional[typing.Dict[str, Parser[Output]]] = None,
+    parsers: Optional[typing.Dict[str, "Parser[Output] | List[Parser[Output]]"]] = None,
     repeated: Optional[Parser[Output]] = None,
 ) -> Callable[[Callable], Callable]:
     """
-    A succinct way to generate a simple `nonpositional` parser. `@command` derives the
+    A succinct way to generate a simple :py:meth:`nonpositional <dollar_lambda.parsers.nonpositional>`
+    parser. :py:func:`@command<command>` derives the
     component parsers from the function's signature and automatically executes the function with
     the parsed arguments, if parsing succeeds:
 
+    >>> from dollar_lambda import command
     >>> @command(help=dict(a="something about a"))
     ... def f(a: int = 1, b: bool = False):
-    ...     return dict(a=a, b=b)
+    ...     print(dict(a=a, b=b))
+    ...
     >>> f("-a", "2", "-b")
     {'a': 2, 'b': True}
 
-    If the wrapped function receives no arguments (as in `f()`), the parser will take
-    `sys.argv[1:]` as the input.
-
-    Note that `@command` does not handle mutually exclusive arguments or alternative
-    arguments.
+    If the wrapped function receives no arguments (as in ``f()``), the parser will take
+    ``sys.argv[1:]`` as the input.
 
     Parameters
     ----------
+
     flip_bools : bool
-        For boolean arguments that default to true, this changes the flag from `--{dest}` to `--no-{dest}`:
+        For boolean arguments that default to true, this changes the flag from ``--{dest}`` to
+        ``--no-{dest}``:
 
     help : dict[str, str]
         A dictionary of help strings for the arguments.
 
+    parsers: Dict[str, Parser | List[Parser]]
+        A dictionary reserving arguments for custom parsers. If the value is a list, the
+        key must correspond to a variadic argument.
+
     repeated: Optional[Parser[Sequence[KeyValue[Any]]]]
         If provided, this parser gets applied repeatedly (zero or more times) at all positions.
 
@@ -103,7 +124,7 @@ def command(
     >>> f("--no-cuda")  # flip_bools adds --no- to the flag
     {'cuda': False}
 
-    As the following example demonstrates, when `flip_bools=False` output can be somewhat confusing:
+    As the following example demonstrates, when ``flip_bools=False`` output can be somewhat confusing:
 
     >>> @command(flip_bools=False)
     ... def f(cuda: bool = True):
@@ -111,7 +132,7 @@ def command(
     >>> f("--cuda")
     {'cuda': False}
 
-    Here is an example using the `help` parameter:
+    Here is an example using the ``help`` parameter:
 
     >>> @command(help=dict(quiet="Be quiet"))
     ... def f(quiet: bool):
@@ -120,9 +141,25 @@ def command(
     usage: --quiet
     quiet: Be quiet
 
-    Here is an example using the `parser` parameter:
+    Here is an example using the ``parsers`` parameter:
 
-    TODO!
+    >>> from dollar_lambda import flag
+    >>> @command(parsers=dict(kwargs=(flag("dev") | flag("prod"))))
+    ... def main(x: int, **kwargs):
+    ...     print(dict(x=x, **kwargs))
+
+    This parser requires either a ``--dev`` or ``--prod`` flag and maps it
+    to the ``kwargs`` argument:
+
+    >>> main("-h")
+    usage: -x X [--dev | --prod]
+    >>> main("-x", "1", "-dev")
+    {'x': 1, 'dev': True}
+    >>> main("-x", "1", "-prod")
+    {'x': 1, 'prod': True}
+    >>> main("-x", "1")
+    usage: -x X [--dev | --prod]
+    The following arguments are required: --dev
     """
 
     def wrapper(func: Callable) -> Callable:
@@ -147,6 +184,42 @@ def command(
     return wrapper
 
 
+class _Function:
+    def __init__(self, callable: Callable, parser: Parser):
+        self.callable = callable
+        self.parser = parser
+
+    def __call__(self, *args: Any, **kwds: Any) -> Any:
+        return self.callable(*args, **kwds)
+
+
+def parser(
+    prefix: Optional[str] = None,
+    flip_bools: bool = True,
+    help: Optional[typing.Dict[str, str]] = None,
+    parsers: Optional[typing.Dict[str, "Parser[Output] | List[Parser[Output]]"]] = None,
+    repeated: Optional[Parser[Output]] = None,
+) -> Callable[[Callable], Callable]:
+    """
+    Adds a ``.parser`` attribute to the wrapped function which can then be used in
+    combination with other parsers. For examples, see: :ref:`GroupingWithParser`.
+    """
+
+    def wrapper(func: Callable) -> Callable:
+        p = _func_to_parser(
+            func,
+            exclude=None,
+            flip_bools=flip_bools,
+            help=help,
+            parsers=parsers,
+            prefix=prefix,
+            repeated=repeated,
+        )
+        return _Function(func, p)
+
+    return wrapper
+
+
 @dataclass
 class _FunctionPair(Monoid[A_co]):
     seq: Sequence[KeyValue[A_co]]
@@ -197,11 +270,6 @@ class _FunctionPair(Monoid[A_co]):
     ) -> Parser[Output["_FunctionPair[str]"]]:
         _help = {} if help is None else help
 
-        # def f(
-        #     _: Sequence[KeyValue[str]],
-        # ) -> Parser[FunctionPair[KeyValue[str]]]:
-        #     return Parser[FunctionPair[KeyValue[str]]](g, usage=usage, helps=_help)
-
         def g(
             cs: Sequence[str],
         ) -> Result[Parse[Output[_FunctionPair[str]]]]:
@@ -229,7 +297,7 @@ class _Node:
     function: Callable
     flip_bools: bool
     help: Optional[typing.Dict[str, str]]
-    parsers: Optional[typing.Dict[str, Parser[Output]]]
+    parsers: Optional[typing.Dict[str, "Parser[Output] | List[Parser[Output]]"]]
     repeated: Optional[Parser[Output]]
     subcommand: bool
     tree: Optional["CommandTree"]
@@ -258,7 +326,7 @@ class _Node:
 class CommandTree:
     """
     Allows parsers to dynamically dispatch their results based on the input. For usage details,
-    see the [`CommandTree` tutorial](#commandtree-tutorial).
+    see the :doc:`command_tree`.
     """
 
     _children: List[_Node] = field(default_factory=list)
@@ -269,7 +337,9 @@ class CommandTree:
         can_run: bool = True,
         flip_bools: bool = True,
         help: Optional[typing.Dict[str, str]] = None,
-        parsers: Optional[typing.Dict[str, Parser[Output]]] = None,
+        parsers: Optional[
+            typing.Dict[str, "Parser[Output] | List[Parser[Output]]"]
+        ] = None,
         repeated: Optional[Parser[Output]] = None,
     ) -> Callable:
         """
@@ -282,7 +352,7 @@ class CommandTree:
             Whether the parser will permit the decorated function to run if no further arguments are supplied.
 
         flip_bools: bool
-            Whether to add `--no-<argument>` before arguments that default to `True`.
+            Whether to add ``--no-<argument>`` before arguments that default to ``True``.
 
         help: dict
             A dictionary of help strings for the arguments.
@@ -290,12 +360,16 @@ class CommandTree:
         repeated: Optional[Parser[Sequence[KeyValue[Any]]]]
             If provided, this parser gets applied repeatedly (zero or more times) at all positions.
 
-        parsers: dict
-            TODO
+        parsers: Dict[str, Parser | List[Parser]]
+            A dictionary reserving arguments for custom parsers. If the value is a list, the
+            key must correspond to a variadic argument.
+            See :py:func:`@command<dollar_lambda.decorators.command>` for examples.
 
         Examples
         --------
-        With `flip_bools` set to `True`:
+        With ``flip_bools`` set to ``True``:
+
+        >>> from dollar_lambda import CommandTree
         >>> tree = CommandTree()
         ...
         >>> @tree.command(flip_bools=True)
@@ -306,7 +380,7 @@ class CommandTree:
         usage: --no-b
         b: (default: True)
 
-        With `flip_bools` set to `False`:
+        With ``flip_bools`` set to ``False``:
 
         >>> tree = CommandTree()
         ...
@@ -319,8 +393,8 @@ class CommandTree:
         b: (default: True)
 
 
-        With `can_run` set to `True` (the default), we can run `f1` by not passing arguments
-        for the `f1`'s children:
+        With ``can_run`` set to ``True`` (the default), we can run ``f1``
+        by not passing arguments for the ``f1``'s children:
 
         >>> tree = CommandTree()
         ...
@@ -334,11 +408,10 @@ class CommandTree:
         ...
         >>> tree("-h")
         usage: -b -n N
-
         >>> tree("-b")
         {'f1': {'b': True}}
 
-        With `can_run` set to `False`, the parser will fail if the child function arguments
+        With ``can_run`` set to ``False``, the parser will fail if the child function arguments
         are not supplied:
 
 
@@ -354,7 +427,6 @@ class CommandTree:
         ...
         >>> tree("-h")
         usage: -b -n N
-
         >>> tree("f1", "-b")
         usage: -b -n N
         Expected '-b'. Got 'f1'
@@ -403,9 +475,9 @@ class CommandTree:
         Run the parser associated with this tree and execute the
         function associated with a succeeding parser.
 
-        If `args` is empty, uses `sys.argv[1:]`.
+        If ``args`` is empty, uses ``sys.argv[1:]``.
         """
-        _args = args if args or parser_mod.TESTING else sys.argv[1:]
+        _args = args if args or parsers.TESTING else sys.argv[1:]
         p = self._parser() >> Parser[Output[_FunctionPair[Any]]].done()
         result = p.parse(Sequence(list(_args))).get
         if isinstance(result, ArgumentError):
@@ -435,22 +507,24 @@ class CommandTree:
             Whether the parser will permit the decorated function to run if no further arguments are supplied.
 
         flip_bools: bool
-            Whether to add `--no-<argument>` before arguments that default to `True`.
+            Whether to add ``--no-<argument>`` before arguments that default to ``True``.
 
         help: Dict[str, str]
             A dictionary of help strings for the arguments.
 
         repeated: Optional[Parser[Sequence[KeyValue[Any]]]]
             If provided, this parser gets applied repeatedly (zero or more times) at all positions.
-            See `nonpositional` for examples.
+            See :py:func:`nonpositional<dollar_lambda.parsers.nonpositional>` for examples.
 
-        parsers: Dict[str, Parser]
-            A dictionary reserving arguments for custom parsers. See below for examples.
-            See `command` for examples.
+        parsers: Dict[str, Parser | List[Parser]]
+            A dictionary reserving arguments for custom parsers. If the value is a list, the
+            key must correspond to a variadic argument.
+            See :py:func:`@command<dollar_lambda.decorators.command>` for examples.
 
         Examples
         --------
-        With `flip_bools` set to `True`:
+        With ``flip_bools`` set to ``True``:
+
         >>> tree = CommandTree()
         ...
         >>> @tree.subcommand(flip_bools=True)
@@ -461,7 +535,7 @@ class CommandTree:
         usage: f1 --no-b
         b: (default: True)
 
-        With `flip_bools` set to `False`:
+        With ``flip_bools`` set to ``False``:
 
         >>> tree = CommandTree()
         ...
@@ -473,8 +547,8 @@ class CommandTree:
         usage: f1 -b
         b: (default: True)
 
-        With `can_run` set to `True` (the default), we can run `f1` by not passing arguments
-        for the `f1`'s children:
+        With ``can_run`` set to ``True`` (the default), we can run ``f1`` by not passing arguments
+        for the ``f1``'s children:
 
         >>> tree = CommandTree()
         ...
@@ -488,11 +562,10 @@ class CommandTree:
         ...
         >>> tree("-h")
         usage: f1 -b g1 -n N
-
         >>> tree("f1", "-b")
         {'f1': {'b': True}}
 
-        With `can_run` set to `False`, the parser will fail if the child function arguments
+        With ``can_run`` set to ``False``, the parser will fail if the child function arguments
         are not supplied:
 
         >>> tree = CommandTree()
@@ -507,7 +580,6 @@ class CommandTree:
         ...
         >>> tree("-h")
         usage: f1 -b g1 -n N
-
         >>> tree("f1", "-b")
         usage: f1 -b g1 -n N
         The following arguments are required: g1
